module CanCan

  module ControllerAdditions

    module ClassMethods
      # 为ActionController::Base设置处理cancan加载资源的处理方法
      def after_loaded_cancan_resource(*args, &block)
        @@_cancan_resource_processors ||= []
        @@_cancan_resource_processors << args
        @@_cancan_resource_processors << block if block
        @@_cancan_resource_processors.flatten!
      end

      # 返回处理cancan被加载资源的处理方法
      def loaded_cancan_resources
        @@_cancan_resource_processors ||= []
      end
    end

    # 返回cancan加载的所有资源
    def cancan_resources
      @_cancan_resources ||= []
    end

  end



  class ControllerResource

    # cancan加载资源后，对该资源进行相关的处理，由扩展的ActionController::Base#after_loaded_cancan_resource设置
    def load_resource_with_handle
      current_ability
      load_resource_without_handle
      load_resource_callback if self.resource_instance
    end
    alias_method_chain(:load_resource, :handle)

    # 扩展的处理加载资源的回调方法调用
    def load_resource_callback
      @controller.cancan_resources << self.resource_instance
      @controller.class.loaded_cancan_resources.each do |processor|
        if processor.is_a?(Proc)
          @controller.instance_eval { processor.call(self.resource_instance) }
        else
          @controller.send(processor, self.resource_instance)
        end
      end
    end

  end

end
